@@ -61,7 +61,8 @@ class CouplingExpression(UserExpression):
 
     def interpolate(self, x):
         # TODO: the correct way to deal with this would be using an abstract class. Since this is technically more
-        #  complex and the current implementation is a workaround anyway, we do not use the proper solution, but this hack.
+        # complex and the current implementation is a workaround anyway, we do not
+        # use the proper solution, but this hack.
         """
         Interpolates at x. Uses buffered interpolant self._f.
         Parameters
@@ -80,7 +81,8 @@ class CouplingExpression(UserExpression):
 
     def create_interpolant(self, x):
         # TODO: the correct way to deal with this would be using an abstract class. Since this is technically more
-        #  complex and the current implementation is a workaround anyway, we do not use the proper solution, but this hack.
+        # complex and the current implementation is a workaround anyway, we do not
+        # use the proper solution, but this hack.
         """
         Creates interpolant from boundary data that has been provided before.
 
@@ -168,12 +170,8 @@ class SegregatedRBFInterpolationExpression(CouplingExpression):
     def segregated_interpolant_2d(self, coords_x, coords_y, data):
         assert(coords_x.shape == coords_y.shape)
         # create least squares system to approximate a * x ** 2 + b * x + c ~= y
-        lstsq_interp = lambda x, y, w: w[0] * x ** 2 + \
-                                       w[1] * y ** 2 + \
-                                       w[2] * x * y + \
-                                       w[3] * x + \
-                                       w[4] * y + \
-                                       w[5]
+
+        def lstsq_interp(x, y, w): return w[0] * x ** 2 + w[1] * y ** 2 + w[2] * x * y + w[3] * x + w[4] * y + w[5]
 
         A = np.empty((coords_x.shape[0], 0))
         n_unknowns = 6
@@ -242,7 +240,7 @@ class EmptyExpression(CouplingExpression):
         :param x: coordinate where expression has to be evaluated
         :param value: buffer where result has to be returned to
         """
-        assert(MPI.size(MPI.comm_world) > 1)
+        assert(MPI.COMM_WORLD.Get_size() > 1)
         for i in range(self._vals.ndim):
             value[i] = 0
 
