@@ -6,9 +6,9 @@ import numpy as np
 from .config import Config
 import logging
 import precice
-from .adapter_core import FunctionType, determine_function_type, convert_fenics_to_precice, set_fenics_vertices, \
-    set_owned_vertices, set_unowned_vertices, get_coupling_boundary_edges, get_forces_as_point_sources, \
-    get_communication_map, communicate_shared_vertices, CouplingMode, Vertices, VertexType
+from .adapter_core import FunctionType, determine_function_type, convert_fenics_to_precice, get_fenics_vertices, \
+    get_owned_vertices, get_unowned_vertices, get_coupling_boundary_edges, get_forces_as_point_sources, \
+    get_communication_map, communicate_shared_vertices, CouplingMode, Vertices, VertexType, filter_point_sources
 from .expression_core import SegregatedRBFInterpolationExpression, EmptyExpression
 from .solverstate import SolverState
 from fenics import Function, FunctionSpace
@@ -51,17 +51,14 @@ class Adapter:
 
         # Setup up MPI communicator on mpi4py
         self._comm = MPI.COMM_WORLD
-        self._rank = self._comm.Get_rank()
-        self._size = self._comm.Get_size()
 
         self._interface = precice.Interface(self._config.get_participant_name(), self._config.get_config_file_name(),
-                                            self._rank, self._size)
+                                            self._comm.Get_rank(), self._comm.Get_size())
 
         # FEniCS related quantities
         self._read_function_space = None  # initialized later
         self._write_function_space = None  # initialized later
         self._dofmap = None  # initialized later using function space provided by user
-        self._coupling_subdomain = None
 
         # coupling mesh related quantities
         self._owned_vertices = Vertices(VertexType.OWNED)
@@ -96,6 +93,20 @@ class Adapter:
         # Determine type of coupling in initialization
         self._coupling_type = None
 
+        # Problem dimension in FEniCS
+        self._fenics_dims = None
+
+    def _is_parallel(self):
+        """
+        Internal function to identify if the adapter is being used for parallel computations
+
+        Returns
+        -------
+        bool : bool
+            True if parallel initialization
+        """
+        return self._comm.Get_size() > 1
+
     def create_coupling_expression(self):
         """
         Creates a FEniCS Expression in the form of an object of class GeneralInterpolationExpression or
@@ -112,21 +123,24 @@ class Adapter:
 
         if not self._empty_rank:
             try:  # works with dolfin 1.6.0
-                coupling_expression = self._my_expression(
-                    element=self._read_function_space.ufl_element())  # element information must be provided, else DOLFIN assumes scalar function
+                # element information must be provided, else DOLFIN assumes scalar function
+                coupling_expression = self._my_expression(element=self._read_function_space.ufl_element())
             except (TypeError, KeyError):  # works with dolfin 2017.2.0
                 coupling_expression = self._my_expression(element=self._read_function_space.ufl_element(), degree=0)
         else:
             try:  # works with dolfin 1.6.0
-                coupling_expression = EmptyExpression(
-                    element=self._read_function_space.ufl_element())  # element information must be provided, else DOLFIN assumes scalar function
+                # element information must be provided, else DOLFIN assumes scalar function
+                coupling_expression = EmptyExpression(element=self._read_function_space.ufl_element())
             except (TypeError, KeyError):  # works with dolfin 2017.2.0
                 coupling_expression = EmptyExpression(element=self._read_function_space.ufl_element(), degree=0)
             if self._read_function_type == FunctionType.SCALAR:
-                coupling_expression._vals = np.empty(
-                    shape=0)  # todo: try to find a solution where we don't have to access the private member coupling_expression._vals
+                # todo: try to find a solution where we don't have to access the private
+                # member coupling_expression._vals
+                coupling_expression._vals = np.empty(shape=0)
             elif self._read_function_type == FunctionType.VECTOR:
-                coupling_expression._vals = np.empty(shape=(0, 0))  # todo: try to find a solution where we don't have to access the private member coupling_expression._vals
+                # todo: try to find a solution where we don't have to access the private
+                # member coupling_expression._vals
+                coupling_expression._vals = np.empty(shape=(0, 0))
 
         coupling_expression.set_function_type(self._read_function_type)
 
@@ -163,14 +177,14 @@ class Adapter:
         Returns
         -------
         x_forces : list
-            List containing X component of forces with reference to respective point sources on the coupling interface.
+            List containing X component of forces with reference to respective point sources on the coupling subdomain.
         y_forces : list
-            List containing Y component of forces with reference to respective point sources on the coupling interface.
+            List containing Y component of forces with reference to respective point sources on the coupling subdomain.
         """
         assert (self._read_function_type is FunctionType.VECTOR), \
             "PointSources only supported for vector valued read data."
 
-        assert (self._size == 1), "get_point_sources function only works in serial."
+        assert (not self._is_parallel()), "get_point_sources function only works in serial."
 
         return get_forces_as_point_sources(self._Dirichlet_Boundary, self._read_function_space, data)
 
@@ -199,7 +213,7 @@ class Adapter:
         read_data = None
 
         if self._empty_rank:
-            assert (self._size > 1)  # having participants without coupling mesh nodes is only valid for parallel runs
+            assert (self._is_parallel()), "having participants without coupling mesh nodes is only valid for parallel runs"
 
         if not self._empty_rank:
             if self._read_function_type is FunctionType.SCALAR:
@@ -208,8 +222,8 @@ class Adapter:
                 read_data = self._interface.read_block_vector_data(read_data_id, self._precice_vertex_ids)
 
             read_data = {tuple(key): value for key, value in zip(self._owned_vertices.get_coordinates(), read_data)}
-            read_data = communicate_shared_vertices(self._comm, self._rank, self._fenics_vertices, self._send_map,
-                                                    self._recv_map, read_data)
+            read_data = communicate_shared_vertices(
+                self._comm, self._fenics_vertices, self._send_map, self._recv_map, read_data)
         else:  # if there are no vertices, we return empty data
             read_data = None
 
@@ -241,7 +255,7 @@ class Adapter:
                                                     self._interface.get_mesh_id(self._config.get_coupling_mesh_name()))
 
         if self._empty_rank:
-            assert (self._size > 1)  # having participants without coupling mesh nodes is only valid for parallel runs
+            assert (self._is_parallel()), "having participants without coupling mesh nodes is only valid for parallel runs"
 
         write_function_type = determine_function_type(write_function)
         assert (write_function_type in list(FunctionType))
@@ -257,7 +271,7 @@ class Adapter:
 
     def initialize(self, coupling_subdomain, read_function_space=None, write_object=None, fixed_boundary=None):
         """
-        Initializes the coupling interface and sets up the mesh in preCICE.
+        Initializes the coupling and sets up the mesh where coupling happens in preCICE.
 
         Parameters
         ----------
@@ -281,10 +295,10 @@ class Adapter:
         """
 
         write_function_space, write_function = None, None
-        if type(write_object) is Function:  # precice.initialize_data() will be called using this Function
+        if isinstance(write_object, Function):  # precice.initialize_data() will be called using this Function
             write_function_space = write_object.function_space()
             write_function = write_object
-        elif type(write_object) is FunctionSpace:  # preCICE will use default zero values for initialization.
+        elif isinstance(write_object, FunctionSpace):  # preCICE will use default zero values for initialization.
             write_function_space = write_object
             write_function = None
         elif write_object is None:
@@ -293,7 +307,7 @@ class Adapter:
             raise Exception("Given write object is neither of type dolfin.functions.function.Function or "
                             "dolfin.functions.functionspace.FunctionSpace")
 
-        if type(read_function_space) is FunctionSpace:
+        if isinstance(read_function_space, FunctionSpace):
             pass
         elif read_function_space is None:
             pass
@@ -336,33 +350,47 @@ class Adapter:
             self._write_function_space = write_function_space
 
         coords = function_space.tabulate_dof_coordinates()
-        _, fenics_dimensions = coords.shape
+        _, self._fenics_dims = coords.shape
 
         # Ensure that function spaces of read and write functions use the same mesh
         if self._coupling_type is CouplingMode.BI_DIRECTIONAL_COUPLING:
-            assert (self._read_function_space.mesh() is write_function_space.mesh()), "read_function_space and " \
-                                                                                     "write_object need to be " \
-                                                                                      "defined using the same mesh"
+            assert (self._read_function_space.mesh() is write_function_space.mesh()
+                    ), "read_function_space and write_object need to be defined using the same mesh"
 
         if fixed_boundary:
             self._Dirichlet_Boundary = fixed_boundary
 
-        if fenics_dimensions != 2:
+        if self._fenics_dims != 2:
             raise Exception("Currently the fenics-adapter only supports 2D cases")
 
-        if fenics_dimensions != self._interface.get_dimensions():
+        if self._fenics_dims != self._interface.get_dimensions():
             raise Exception("Dimension of preCICE setup and FEniCS do not match")
 
-        # Set vertices on the coupling interface for this rank
-        set_fenics_vertices(function_space, coupling_subdomain, self._fenics_vertices)
-        set_owned_vertices(function_space, coupling_subdomain, self._owned_vertices)
-        set_unowned_vertices(function_space, coupling_subdomain, self._unowned_vertices)
+        # Set vertices on the coupling subdomain for this rank
+        lids, gids, coords = get_fenics_vertices(function_space, coupling_subdomain, self._fenics_dims)
+        self._fenics_vertices.set_local_ids(lids)
+        self._fenics_vertices.set_global_ids(gids)
+        self._fenics_vertices.set_coordinates(coords)
+
+        if self._is_parallel():
+            lids, gids, coords = get_owned_vertices(function_space, coupling_subdomain, self._fenics_dims)
+            self._owned_vertices.set_local_ids(lids)
+            self._owned_vertices.set_global_ids(gids)
+            self._owned_vertices.set_coordinates(coords)
+
+            gids = get_unowned_vertices(function_space, coupling_subdomain, self._fenics_dims)
+            self._unowned_vertices.set_global_ids(gids)
+        else:
+            # For serial execution, owned vertices are identical to fenics vertices
+            self._owned_vertices.set_local_ids(lids)
+            self._owned_vertices.set_global_ids(gids)
+            self._owned_vertices.set_coordinates(coords)
 
         # Set up mesh in preCICE
         if self._fenics_vertices.get_global_ids().size > 0:
             self._empty_rank = False
         else:
-            print("Rank {} has no part of coupling boundary.".format(self._rank))
+            print("Rank {} has no part of coupling boundary.".format(self._comm.Get_rank()))
 
         # Define mesh in preCICE
         self._precice_vertex_ids = self._interface.set_mesh_vertices(self._interface.get_mesh_id(
@@ -371,11 +399,18 @@ class Adapter:
         if self._coupling_type is CouplingMode.UNI_DIRECTIONAL_READ_COUPLING or \
                 self._coupling_type is CouplingMode.BI_DIRECTIONAL_COUPLING:
             # Determine shared vertices with neighbouring processes and get dictionaries for communication
-            self._send_map, self._recv_map = get_communication_map(self._comm, self._rank, self._read_function_space,
-                                                                   self._owned_vertices, self._unowned_vertices)
+            self._send_map, self._recv_map = get_communication_map(self._comm, self._read_function_space,
+                                                                   self._owned_vertices,
+                                                                   self._unowned_vertices)
 
-        # # Set mesh edges in preCICE to allow nearest-projection mapping
-        # # Define a mapping between coupling vertices and their IDs in preCICE
+        # Check for double boundary points
+        if fixed_boundary:
+            # create empty data for the sake of searching for duplicate points
+            point_data = {tuple(key): None for key in self._owned_vertices.get_coordinates()}
+            _ = filter_point_sources(point_data, fixed_boundary, warn_duplicate=True)
+
+        # Set mesh edges in preCICE to allow nearest-projection mapping
+        # Define a mapping between coupling vertices and their IDs in preCICE
         id_mapping = {key: value for key, value in zip(self._owned_vertices.get_global_ids(), self._precice_vertex_ids)}
 
         edge_vertex_ids1, edge_vertex_ids2 = get_coupling_boundary_edges(function_space, coupling_subdomain,
@@ -464,7 +499,7 @@ class Adapter:
 
     def finalize(self):
         """
-        Completes the coupling interface execution. To be called at the end of the simulation.
+        Finalizes the coupling via preCICE and the adapter. To be called at the end of the simulation.
 
         Notes
         -----
